﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Contracts;
using System.ServiceModel;
using WcfContrib.Client;

namespace WcfContrib.Showcase.Client
{
    class Program
    {
        static void Main(string[] args)
        {
            //load runtime, if not called will be called automatically upon first access to the client channel
            ClientChannel<IService>.LoadRuntime();

            //set the endpoint name, here we do it by setting the global definition - for all these client contract instances
            ClientChannel<IService>.GlobalEndpointName = "ServiceEP";

            ////////////////////////////////
            //NEW FROM 2.0:
            //This simple set increases the max items in object graph of the client endpoint as well as boosts the binding - increases max received message size, buffer sizes and reader quotas.
            ////////////////////////////////
            ////////////////////////////////
            //NEW FROM 2.1.0.0:
            //You can define the apply boosting in the new configuration section and it will be applied to all client channels unless specified otherwise per-instance
            ////////////////////////////////
            ClientChannel<IService>.Instance.ApplyBoosting = true;

            while (true)
            {
                Console.ForegroundColor = ConsoleColor.Cyan;
                Console.WriteLine("Press any key to make a call to the service, 'Enter' to exit'");

                if (Console.ReadKey().Key == ConsoleKey.Enter)
                {
                    break;
                }

                try
                {
                    TestClientCall();

                    TestSeveralClientCalls();

                    Console.ForegroundColor = ConsoleColor.White;
                    Console.WriteLine("Service was called successfully");
                }
                catch (Exception ex)
                {
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine("Error: " + ex.ToString());
                }
            }

            //Dispose the client channel instance when exiting
            ClientChannel<IService>.Instance.Dispose();
        }

        static void TestClientCall()
        {
            //we call the service
            ClientChannel<IService>.Instance.Channel.Do(new Foo { Id = 1, Name = "Some Foo" });
        }

        static void TestSeveralClientCalls()
        {
            ClientChannel<IService> client = new ClientChannel<IService>(ChannelManageOptions.Sequential);

            client.Channel.Do(new Foo { Id = 1, Name = "Some Foo" });
            client.Channel.Do(new Foo { Id = 1, Name = "Some Foo" });

            client.Dispose();
        }
    }
}
