﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Contracts;
using System.ServiceModel;
using WcfContrib.Client;

namespace WcfContrib.Showcase.Client
{
    class Program
    {
        static void Main(string[] args)
        {
            //Choose environment, note - this affects all client channels unless tagged with ignoring endpoint aggregation
            ClientModel.EndpointAggregation = "_Development";

            //load runtime, if not called will be called automatically upon first access to the client channel
            ClientChannel<IService>.LoadRuntime();

            //set the endpoint name, here we do it by setting the global definition - for all these client contract instances
            ClientChannel<IService>.GlobalEndpointName = "ServiceEP";

            while (true)
            {
                Console.ForegroundColor = ConsoleColor.Cyan;
                Console.WriteLine("Press any key to make a call to the service, 'Enter' to exit'");

                if (Console.ReadKey().Key == ConsoleKey.Enter)
                {
                    break;
                }

                try
                {
                    TestClientCall();

                    Console.ForegroundColor = ConsoleColor.White;
                    Console.WriteLine("Service was called successfully");
                }
                catch (Exception ex)
                {
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine("Error: " + ex.ToString());
                }
            }

            //Dispose the client channel instance when exiting
            ClientChannel<IService>.Instance.Dispose();
        }

        static void TestClientCall()
        {
            //can set an instance of a client channel to ignore the endpoint aggregation if needed
            //ClientChannel<IService>.Instance.IgnoreEndpointAggregation = true;

            ClientChannel<IService>.Instance.Channel.Do(new Foo { Id = 1, Name = "Some Foo" });
        }
    }
}
