﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Contracts;
using System.ServiceModel;

namespace WcfContrib.Showcase.Client
{
    class Program
    {
        static void Main(string[] args)
        {
            while (true)
            {
                Console.ForegroundColor = ConsoleColor.Cyan;
                Console.WriteLine("Press any key to make a call to the service, 'Enter' to exit'");

                if (Console.ReadKey().Key == ConsoleKey.Enter)
                {
                    break;
                }

                try
                {
                    Console.ForegroundColor = ConsoleColor.White;
                    
                    TestClientCall(GetChannel("ServiceEP"));
                    Console.WriteLine("");
                    TestClientCallWithTrace(GetChannel("ServiceEPTrace"));
                    Console.WriteLine("");
                    TestClientCallWithMock(GetChannel("ServiceEPMock"));
                    Console.WriteLine("");
                }
                catch (Exception ex)
                {
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine("Error: " + ex.ToString());
                }
            }
        }

        static void TestClientCall(IService channel)
        {
            channel.Do(new Foo { Id = 1, Name = "Some Foo" });
            CloseChannel(channel);

            Console.WriteLine("-- Service was called successfully");
        }

        static void TestClientCallWithMock(IService channel)
        {
            channel.Do2(new Foo { Id = 1, Name = "Some Foo" });
            CloseChannel(channel);

            Console.WriteLine("-- Service was called with mock and trace (see that the service was called but the operation itself wasn't invoked)");
        }

        static void TestClientCallWithTrace(IService channel)
        {
            channel.Do2(new Foo { Id = 1, Name = "Some Foo" });
            CloseChannel(channel);

            Console.WriteLine("-- Service was called with trace. (check the log file in the service folder)");
        }

        #region Channel Helpers
        static IService GetChannel(string name)
        {
            ChannelFactory<IService> factory = new ChannelFactory<IService>(name);

            return factory.CreateChannel();
        }

        static void CloseChannel(IService channel)
        {
            ((ICommunicationObject)channel).Close();
        }
        #endregion
    }
}
