﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Contracts;
using System.ServiceModel;
using WcfContrib.Client;
using WcfContrib.Helpers;
using System.Threading;
using System.Reflection;
using WcfContrib.Extensions.Caching;
using WcfContrib.Client.Behaviors;

namespace WcfContrib.Test.Client
{
    class Program
    {
        static void Main(string[] args)
        {
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
            //bool ol = EndpointHelper.IsOnline("http://localhost:2025/SampleServicesRealm", "http://localhost:2025/SampleServicesRealm");
            //bool o2 = EndpointHelper.IsOnline("http://localhost:2025/SampleServicesRealmNONE", "http://localhost:2025/SampleServicesRealm");
            //bool o3 = EndpointHelper.IsOnline("http://localhost:2025/SampleServicesRealm", "http://localhost:2025/SampleServicesRealmNONE");
            //bool o4 = EndpointHelper.IsOnline("http://localhost:2025/SampleServicesRealmNONE", "http://localhost:2025/SampleServicesRealmNONE");

            //Usage of Base Client Channels
            ChannelFactory<IService> myFactory = new ChannelFactory<IService>(
                new NetTcpBinding(), new EndpointAddress("net.tcp://localhost:11000/MyService"));
            using (MyClientChannel<IService> myClientChannel = new MyClientChannel<IService>(myFactory))
            {
                myClientChannel.Channel.Do(null);
            }

            ClientChannel<IService>.GlobalEndpointName = "ServiceNetTcpEP";
            ClientChannel<IService>.LoadRuntime();

            //IService channel = GetChannel();
            MyClient clientChannel = MyClient.Instance;
            //MyClient clientChannel = new MyClient(ChannelManageOptions.Sequential);
            int runCount = 1;

            while (true)
            {
                Console.ForegroundColor = ConsoleColor.Cyan;
                Console.WriteLine("Press any key to make a call to the service, 'Enter' to exit'");

                if (Console.ReadKey().Key == ConsoleKey.Enter)
                {
                    break;
                }

                try
                {
                    if (runCount <= 1)
                    {
                        TestClientCall(clientChannel);

                        //if (ClientModel.EndpointAggregation != null && ClientModel.EndpointAggregation.EndsWith("1"))
                        //{
                        //    ClientModel.EndpointAggregation = string.Empty;
                        //}
                        //else
                        //{
                        //    ClientModel.EndpointAggregation += "1";
                        //}
                    }
                    else
                    {
                        var handles = Enumerable.Repeat(new Action(() => TestClientCall(clientChannel)), runCount)
                            .Select(a => a.BeginInvoke(null, null).AsyncWaitHandle).ToArray();

                        WaitHandle.WaitAll(handles);
                    }

                    Console.ForegroundColor = ConsoleColor.Green;
                    Console.WriteLine("Service was called successfully");
                }
                catch (Exception ex)
                {
                    //channel = GetChannel();

                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine("Error: " + ex.ToString());
                }
            }

            using (OutputCacheController.Current) { }
            using (clientChannel) { }
            //closeChannel(channel);

            Console.WriteLine("Press any key to exit.");
            Console.ReadKey();
        }

        static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            Helpers.WriteLine(ConsoleColor.Magenta, "CurrentDomain_UnhandledException: {0}", e.ExceptionObject);
        }

        static void TestClientCall(MyClient channel)
        {
            try
            {
                //var bChannel = GetChannel();
                //bChannel.BeginGetFoo1(new AsyncCallback(OnBuiltInAsyncComplete), bChannel);

                IAsyncResult r = null;
                Foo f1 = null;
                Foo f = new Foo { Id = 1, Name = "Some Foo" };

                //channel.Channel.Do(f);
                //Helpers.WriteLine(ConsoleColor.White, "Do: done");

                //r = channel.Channel.BeginGetFoo1(new AsyncCallback(OnOperationComplete), null);
                //r = channel.Channel.BeginGetFoo2(f.Id, new AsyncCallback(OnOperationComplete), null);
                //r = channel.Channel.BeginGetFoo2(f.Id, new AsyncCallback(OnOperationComplete), null);

                //r = channel.GetFoo1Async(new AsyncCallback(OnOperationCallback), null);
                //r = channel.GetFoo2Async(f.Id, new AsyncCallback(OnOperationCallback), null);

                //f = channel.Channel.GetFoo1();

                channel.Channel.BeginDo(f, OnOperationComplete, null);
                //channel.Channel.BeginDo(f, OnOperationComplete, null);
                //f = channel.Channel.GetFoo1();

                //r = channel.Channel.BeginGetFoo1(new AsyncCallback(OnOperationComplete), null);

                #region Output Cache
                //Output Cache Test:

                //f1 = channel.Channel.GetFoo1();
                //if (f1 == null)
                //{
                //    Helpers.WriteLine(ConsoleColor.White, "GetFoo1: null");
                //}
                //else
                //{
                //    Helpers.WriteLine(ConsoleColor.White, "GetFoo1: Foo Id - {0}", f1.Id);
                //}

                //f1 = channel.Channel.GetFoo2(f.Id);
                //Helpers.WriteLine(ConsoleColor.White, "GetFoo2: Foo Id - {0}", f1.Id);

                //f1 = channel.Channel.GetFoo3(new[] { 1, 2 });
                //Helpers.WriteLine(ConsoleColor.White, "GetFoo3: Foo Id - {0}", f1.Id);

                //f1 = channel.Channel.GetFoo4(new[] { f, f });
                //Helpers.WriteLine(ConsoleColor.White, "GetFoo4: Foo Id - {0}", f1.Id);

                //f1 = channel.Channel.GetFoo5(new Foo { Id = f.Id, Name = f.Name });
                //Helpers.WriteLine(ConsoleColor.White, "GetFoo5: Foo Id - {0}", f1.Id);
                #endregion
            }
            catch (Exception ex)
            {
                Helpers.WriteLine(ConsoleColor.Red, "TestClientCall Error: {0}", ex);
            }
        }

        static void OnOperationComplete(IAsyncResult result)
        {
            ChannelInvokeAsyncResult r = result as ChannelInvokeAsyncResult;
            
            if (r.HasError)
            {
                Helpers.WriteLine(ConsoleColor.Red, "OnOperationComplete: Error - {0} - {1}\r\n\r\n", r.Exception.GetType().Name, r.Exception.Message);
            }
            else if (r.Result == null)
            {
                Helpers.WriteLine(ConsoleColor.White, "OnOperationComplete: Result is null");
            }
            else
            {
                Helpers.WriteLine(ConsoleColor.White, "OnOperationComplete: {0}", r.Result);
            }
        }

        static void OnBuiltInAsyncComplete(IAsyncResult result)
        {
            IService channel = result.AsyncState as IService;

            var foo = channel.EndGetFoo1(result);
            Helpers.WriteLine(ConsoleColor.White, "OnBuiltInAsyncComplete: Foo Id - {0}", foo.Id);

            try
            {
                ((ICommunicationObject)channel).Close();
            }
            catch (Exception)
            {
                ((ICommunicationObject)channel).Abort();
            }
        }

        static void OnOperationCallback(IAsyncResult result)
        {
            ChannelInvokeAsyncResult r = result as ChannelInvokeAsyncResult;

            if (r.HasError)
            {
                Helpers.WriteLine(ConsoleColor.Red, "OnOperationCallback: Error - {0}\r\n\r\n", r.Exception.Message);
            }
            else if (r.Result == null)
            {
                Helpers.WriteLine(ConsoleColor.White, "OnOperationCallback: Result is null");
            }
            else
            {
                Helpers.WriteLine(ConsoleColor.White, "OnOperationComplete: {0}", r.Result);
            }
        }

        #region Channel Helpers
        static IService GetChannel()
        {
            ChannelFactory<IService> factory = new ChannelFactory<IService>("ServiceNetTcpEP");

            return factory.CreateChannel();
        }

        static void CloseChannel(IService channel)
        {
            ((ICommunicationObject)channel).Close();
        }
        #endregion
    }

    class MyClient : ClientChannel<MyClient, IService>
    {
        public MyClient(ChannelManageOptions managementOption) : base(managementOption)
        {

        }

        public MyClient()
        {

        }

        protected override void InitializeRuntime()
        {
            base.InitializeRuntime();

            Description.ChannelHandlers.Add(new MyChannelInitExtension());
        }

        public IAsyncResult GetFoo1Async(AsyncCallback callback, object asyncState)
        {
            return InvokeChannelAsync(MethodBase.GetCurrentMethod(), callback, asyncState);
        }

        public IAsyncResult GetFoo2Async(int id, AsyncCallback callback, object asyncState)
        {
            return InvokeChannelAsync(MethodBase.GetCurrentMethod(), callback, asyncState, id);
        }

        protected override InvokeErrorAction OnInvokeException(ChannelExceptionEventArgs args)
        {
            base.OnInvokeException(args);
            
            //return InvokeErrorAction.Suppress;
            return InvokeErrorAction.Throw;
        }
    }

    class MyChannelInitExtension : IClientChannelHandlerBehavior
    {
        public void ChannelFactoryCreated(ClientChannel clientChannel, ChannelFactory factory)
        {
            Helpers.WriteLine(ConsoleColor.Gray, "MyChannelInitExtension.ChannelFactoryCreated");
        }

        public void ChannelCreated(ClientChannel clientChannel, ICommunicationObject channel)
        {
            Helpers.WriteLine(ConsoleColor.Gray, "MyChannelInitExtension.ChannelCreated");
        }
    }

    static class Helpers
    {
        internal static void WriteLine(ConsoleColor color, string input, params object[] format)
        {
            ConsoleColor orgColor = Console.ForegroundColor;

            Console.ForegroundColor = color;
            if (format.Length > 0)
            {
                Console.WriteLine(string.Format(input, format));
            }
            else
            {
                Console.WriteLine(input);
            }

            Console.ForegroundColor = orgColor;
        }
    }
}
