﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WcfContrib.Hosting;
using System.ServiceModel;

namespace WcfContrib.Test.Services
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                ServiceHost<Service> h = new ServiceHost<Service>();

                ServiceHost h2 = ServiceConfigurationDescription.Create("SampleServicesRealm")
                    .WithBasicHttp(2025)
                    .WithBasicHttp("anon", KnownSecurityMode.None)
                    .WithNetTcp(2024, KnownEndpointConfiguration.NetTcp)
                    .WithNetTcp("rs", KnownEndpointConfiguration.NetTcpReliableSession)
                    .WithWsHttp("ws")
                    .WithWebHttp("web", KnownEndpointConfiguration.WebHttp)
                    .MakeDiscoverable()
                    .GenerateServiceHost<Service2>(s => s.ApplyBoosting = true);

                h.Open();
                h2.Open();

                string eps = h2.Description.Endpoints.Select(
                    e => string.Format("Endpoint: Address - {0} / Binding - {1}", e.Address, e.Binding.GetType().Name)).Aggregate((s1, s2) => s1 + "\r\n" + s2);

                Console.WriteLine("Service is up, press 'Enter to exit'");
                Console.WriteLine("\r\n{0}", eps);
                Console.ReadLine();
                
                h.Close();
                h2.Close();
            }
            catch (Exception ex)
            {
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("Error: " + ex.ToString());
                Console.ReadLine();
            }
        }
    }

    static class Helpers
    {
        internal static void WriteLine(ConsoleColor color, string input, params object[] format)
        {
            ConsoleColor orgColor = Console.ForegroundColor;

            Console.ForegroundColor = color;
            if (format.Length > 0)
            {
                Console.WriteLine(string.Format(input, format));
            }
            else
            {
                Console.WriteLine(input);
            }

            Console.ForegroundColor = orgColor;
        }
    }
}
